//
//  ViewController.swift
//  FrequencyCounter
//
//  Created by Phidgets on 2018-01-29.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {
    
    let ch = FrequencyCounter()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    @IBOutlet var countLabel: UILabel!
    @IBOutlet var timeLabel: UILabel!
    @IBOutlet var frequencyLabel: UILabel!
    @IBOutlet var totalCountLabel: UILabel!
    @IBOutlet var totalTimeLabel: UILabel!
    @IBOutlet var enabledStack: UIStackView!
    @IBOutlet var enabledSwitch: UISwitch!
    @IBOutlet var frequencyCutoffTextfield: UITextField!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var powerSupplySegmentedControl: UISegmentedControl!
    @IBOutlet var inputModeSegmentedControl: UISegmentedControl!
    @IBOutlet var daq1400Stack: UIStackView!
    @IBOutlet var filterTypeStack: UIStackView!
    @IBOutlet var stackView: UIStackView!
    
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.frequencyChange.addHandler(frequencychange_handler)
            let _ = ch.countChange.addHandler(countchange_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }

    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let attachedDevice = sender as! FrequencyCounter
            
            var powerSupply:PowerSupply? = nil
            var inputMode:InputMode? = nil
            let deviceID = try attachedDevice.getDeviceID()
            if(deviceID == DeviceID.PN_DAQ1400){
                powerSupply = try attachedDevice.getPowerSupply()
                inputMode = try attachedDevice.getInputMode()
            }
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            let isEnabled = try attachedDevice.getEnabled()
            let frequencyCutoff = try attachedDevice.getFrequencyCutoff()
            
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = String(format: "%d ms", UInt32(dataInterval))
                self.frequencyCutoffTextfield.text = String(format:"%.2f",frequencyCutoff)
                self.enabledSwitch.isEnabled = isEnabled
                if(deviceID == DeviceID.PN_1054){
                    self.enabledStack.isHidden = false
                    self.daq1400Stack.isHidden = true
                    self.filterTypeStack.isHidden = false
                }
                else if(deviceID == DeviceID.PN_DAQ1400){
                    self.enabledStack.isHidden = true
                    self.filterTypeStack.isHidden = true
                    self.daq1400Stack.isHidden = false
                    self.powerSupplySegmentedControl.selectedSegmentIndex = (powerSupply == PowerSupply.off) ? 0 : (powerSupply == PowerSupply.volts_12) ? 1 : 2
                    self.inputModeSegmentedControl.selectedSegmentIndex = (inputMode == InputMode.NPN) ? 0 : 1
                }
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func frequencychange_handler(sender: FrequencyCounter, frequency: Double){
        DispatchQueue.main.async {
            self.frequencyLabel.text = String(format:"%f Hz", frequency)
        }
    }
    
    func countchange_handler(sender: FrequencyCounter, data: (counts: UInt64, timeChange: Double)){
        do{
            let totalCount = try ch.getCount()
            let totalTime = try ch.getTimeElapsed()
            DispatchQueue.main.async {
                self.countLabel.text = "\(data.counts)"
                self.timeLabel.text = "\(data.timeChange) ms"
                self.totalTimeLabel.text = "\(totalTime)"
                self.totalCountLabel.text = "\(totalCount)"
            }
        }catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: GUI Controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            DispatchQueue.main.async {
                self.dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setFrequencyCutoff(_ sender: Any) {
        do{
            self.view.endEditing(true)
            var frequencyCutoff = Double(frequencyCutoffTextfield.text!)
            if(frequencyCutoff != nil){
                try ch.setFrequencyCutoff(frequencyCutoff!)
            }
            else{
                frequencyCutoff = try ch.getFrequencyCutoff()
                let alert = UIAlertController(title: "Error", message: "Frequency Cutoff Incorrect Format", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
                }))
                self.present(alert, animated: true, completion: nil)
                DispatchQueue.main.async {
                    self.frequencyCutoffTextfield.text = String(format:"%.2f",frequencyCutoff!)
                }
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setEnabled(_ sender: Any) {
        let swtch = sender as! UISwitch
        do{
            try ch.setEnabled(swtch.isOn)
        }catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setFilterType(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setFilterType(FrequencyFilterType.zeroCrossing)
                break
            case 1:
                try ch.setFilterType(FrequencyFilterType.logicLevel)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setPowerSupply(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setPowerSupply(PowerSupply.off)
                break
            case 1:
                try ch.setPowerSupply(PowerSupply.volts_12)
                break
            case 2:
                try ch.setPowerSupply(PowerSupply.volts_24)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setInputMode(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setInputMode(InputMode.NPN)
                break
            case 1:
                try ch.setInputMode(InputMode.PNP)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    //MARK: navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }

}

